#### Sample Code to Implement Proportion, Count, and Scaling Tests from Jiao and Pretis "Testing the Presence of Outliers in Regression Models"
#https://papers.ssrn.com/sol3/papers.cfm?abstract_id=3217213

#install.packages("gets)
library(gets)

##################################################################
############# 'Simple' Proportion and Count Tests ##############
##################################################################

####################################
#### Example 1: running Impulse Indicator Saturation within R then testing the proportion and number of outliers.
####################################

# simulate data without extra outliers
set.seed(123)
y <- rnorm(100, 0, 1)

# simulate data with outliers
yo <- y
yo[c(10, 17, 55, 84, 95)] <- yo[c(10, 17, 55, 84, 95)] + 4 #add 5 artificial positive outliers

is1 <- isat(y, iis=TRUE, sis=FALSE, t.pval=0.01) # expect 1% of observations as outliers under the null of no outliers
is2 <- isat(yo, iis=TRUE, sis=FALSE, t.pval=0.01) # expect 1% of observations as outliers under the null of no outliers

outliertest(is1) # null hypothesis is not rejected, observed proportion is similar to expected
outliertest(is2) # null hypothesis is rejected, observed proportion is different to expected

## One sided tests:
outliertest(is1, alternative = "greater") 
outliertest(is2, alternative = "greater")

#################################
#### Example 2: Testing the proportion and number of outliers using existing IIS estimates (such as coming from PcGive)
#################################


#Hypothetical example:
obs <- 100 #number of observations
expected <- 0.01 #expected proportion of outliers, i.e. level of significance of selection
observed1 <- 0.02 #observed proportion of outliers from estimation
observed2 <- 0.05 #observed proportion of outliers from estimation

### number of outliers
n1 <- observed1*obs
n2 <- observed2*obs

outliertest(noutl = n1, t.pval=expected, T=obs, alternative = "two.sided") #do not reject the null hypothesis, p-value of proportion test is 0.236
outliertest(noutl = n2, t.pval=expected, T=obs, alternative = "two.sided") #reject the null hypothesis, p-value of proportion test is 2.152e-06


##################################################################
############# Scaling Tests ##############
##################################################################

# simulate data without extra outliers
set.seed(123)
y <- rnorm(100, 0, 1)

# simulate data with outliers
yo <- y
yo[c(10, 17, 55, 84, 95)] <- yo[c(10, 17, 55, 84, 95)] + 4 #add 5 artificial positive outliers

isloop1 <- isatloop(y=y, iis=TRUE, sis=FALSE)
isloop2 <- isatloop(y=yo, iis=TRUE, sis=FALSE)

outlierscaletest(isloop1) #null hypothesis is not rejected using the Sum and Sup tests
outlierscaletest(isloop2) #null hypothesis is rejected using the Sum and Sup tests
